import itertools

from django.db.models.lookups import Lookup, GreaterThan, GreaterThanOrEqual, LessThan, LessThanOrEqual
from django.utils.datastructures import OrderedSet
from django.core.exceptions import EmptyResultSet

from .hashid import Hashid, _is_int
from .conf import settings


def get_id_for_hashid_field(field, value):
    if isinstance(value, Hashid):
        return value.id
    try:
        hashid = field.get_hashid(value)
    except ValueError:
        raise ValueError(field.error_messages['invalid'] % {'value': value})
    if not field.allow_int_lookup and _is_int(value):
        raise ValueError(field.error_messages['invalid_hashid'] % {'value': value})
    return hashid.id


# Most of this code is derived or copied from Django. (django/db/models/lookups.py)
# It has been included here to increase compatibility of this module with Django versions 1.11, 2.2 and 3.0.
# Django is Copyright (c) Django Software Foundation and individual contributors.
# Please see https://github.com/django/django/blob/master/LICENSE

class HashidFieldGetDbPrepValueMixin:
    get_db_prep_lookup_value_is_iterable = False

    def get_db_prep_lookup(self, value, connection):
        # There are two modes this method can be called in... a single value or an iterable of values (usually a set)
        # For a single value, just try to process it, then return the value, or else throw EmptyResultSet
        # For multiple values, process each one in turn. If any of them are invalid, throw it away. If all are invalid,
        # throw EmptyResultSet
        # For relational fields, use the 'field' attribute of the output_field
        field = getattr(self.lhs.output_field, 'field', self.lhs.output_field)
        if self.get_db_prep_lookup_value_is_iterable:
            lookup_ids = []
            for val in value:
                try:
                    lookup_id = get_id_for_hashid_field(field, val)
                except ValueError:
                    if settings.HASHID_FIELD_LOOKUP_EXCEPTION:
                        raise
                    # Ignore this value
                    pass
                else:
                    lookup_ids.append(lookup_id)
            if len(lookup_ids) == 0:
                raise EmptyResultSet
            return '%s', lookup_ids
        else:
            try:
                lookup_id = get_id_for_hashid_field(field, value)
            except ValueError:
                if settings.HASHID_FIELD_LOOKUP_EXCEPTION:
                    raise
                raise EmptyResultSet
            return '%s', [lookup_id]


class HashidExactLookup(HashidFieldGetDbPrepValueMixin, Lookup):
    prepare_rhs = False

    def as_sql(self, compiler, connection):
        lhs_sql, params = self.process_lhs(compiler, connection)
        rhs_sql, rhs_params = self.process_rhs(compiler, connection)
        params.extend(rhs_params)
        rhs_sql = self.get_rhs_op(connection, rhs_sql)
        return '%s %s' % (lhs_sql, rhs_sql), params

    def get_rhs_op(self, connection, rhs):
        return "= %s" % rhs


class HashidIterableLookup(HashidExactLookup):
    # This is an amalgamation of Django's FieldGetDbPrepValueIterableMixin and In lookup to allow support of both
    # iterables (lists, tuples) and subqueries.
    get_db_prep_lookup_value_is_iterable = True

    def get_prep_lookup(self):
        if hasattr(self.rhs, 'resolve_expression'):
            return self.rhs
        prepared_values = []
        if hasattr(self.rhs, '_prepare'):
            # A subquery is like an iterable but its items shouldn't be
            # prepared independently.
            return self.rhs._prepare(self.lhs.output_field)
        for rhs_value in self.rhs:
            if hasattr(rhs_value, 'resolve_expression'):
                # An expression will be handled by the database but can coexist
                # alongside real values.
                pass
            prepared_values.append(rhs_value)
        return prepared_values

    def process_rhs(self, compiler, connection):
        db_rhs = getattr(self.rhs, '_db', None)
        if db_rhs is not None and db_rhs != connection.alias:
            raise ValueError(
                "Subqueries aren't allowed across different databases. Force "
                "the inner query to be evaluated using `list(inner_query)`."
            )

        if self.rhs_is_direct_value():
            try:
                rhs = OrderedSet(self.rhs)
            except TypeError:  # Unhashable items in self.rhs
                rhs = self.rhs

            if not rhs:
                raise EmptyResultSet

            # rhs should be an iterable; use batch_process_rhs() to
            # prepare/transform those values.
            sqls, sqls_params = self.batch_process_rhs(compiler, connection, rhs)
            placeholder = '(' + ', '.join(sqls) + ')'
            return (placeholder, sqls_params)
        else:
            return super().process_rhs(compiler, connection)

    def resolve_expression_parameter(self, compiler, connection, sql, param):
        params = [param]
        if hasattr(param, 'resolve_expression'):
            param = param.resolve_expression(compiler.query)
        if hasattr(param, 'as_sql'):
            sql, params = param.as_sql(compiler, connection)
        return sql, params

    def batch_process_rhs(self, compiler, connection, rhs=None):
        pre_processed = super().batch_process_rhs(compiler, connection, rhs)
        # The params list may contain expressions which compile to a
        # sql/param pair. Zip them to get sql and param pairs that refer to the
        # same argument and attempt to replace them with the result of
        # compiling the param step.
        sql, params = zip(*(
            self.resolve_expression_parameter(compiler, connection, sql, param)
            for sql, param in zip(*pre_processed)
        ))
        params = itertools.chain.from_iterable(params)
        return sql, tuple(params)

    def get_rhs_op(self, connection, rhs):
        return 'IN %s' % rhs


class HashidGreaterThan(HashidFieldGetDbPrepValueMixin, GreaterThan):
    prepare_rhs = False


class HashidGreaterThanOrEqual(HashidFieldGetDbPrepValueMixin, GreaterThanOrEqual):
    prepare_rhs = False


class HashidLessThan(HashidFieldGetDbPrepValueMixin, LessThan):
    prepare_rhs = False


class HashidLessThanOrEqual(HashidFieldGetDbPrepValueMixin, LessThanOrEqual):
    prepare_rhs = False



from hashids import Hashids

from .hashid import Hashid


class HashidDescriptor(object):
    def __init__(self, field_name, hashids, prefix="", enable_hashid_object=True):
        self.field_name = field_name
        self.hashids = hashids
        self.prefix = prefix
        self.enable_hashid_object = enable_hashid_object

    def __get__(self, instance, owner=None):
        if instance is not None and self.field_name in instance.__dict__:
            return instance.__dict__[self.field_name]
        else:
            return None

    def __set__(self, instance, value):
        self._set_value(instance, self.field_name, value, enable_hashid_object=self.enable_hashid_object)
        if not self.enable_hashid_object:
            self._set_value(instance, self.field_name + "_hashid", value, enable_hashid_object=True)

    def _set_value(self, instance, name, value, enable_hashid_object):
        if value is None:
            instance.__dict__[name] = value
        if isinstance(value, Hashid):
            if enable_hashid_object:
                instance.__dict__[name] = value
            else:
                instance.__dict__[name] = str(value)
        else:
            try:
                h = Hashid(value, prefix=self.prefix, hashids=self.hashids)
                if enable_hashid_object:
                    instance.__dict__[name] = h
                else:
                    instance.__dict__[name] = str(h)
            except ValueError:
                instance.__dict__[name] = value



from django.apps import apps
from django.core import exceptions

from hashids import Hashids
from rest_framework import fields, serializers

from hashid_field.conf import settings
from hashid_field.hashid import Hashid


class UnconfiguredHashidSerialField(fields.Field):
    def bind(self, field_name, parent):
        super().bind(field_name, parent)
        raise exceptions.ImproperlyConfigured(
            "The field '{field_name}' on {parent} must be explicitly declared when used with a ModelSerializer".format(
                field_name=field_name, parent=parent.__class__.__name__))


class HashidSerializerMixin(object):
    usage_text = "Must pass a HashidField, HashidAutoField or 'app_label.model.field'"

    def __init__(self, **kwargs):
        self.hashid_salt = kwargs.pop('salt', settings.HASHID_FIELD_SALT)
        self.hashid_min_length = kwargs.pop('min_length', 7)
        self.hashid_alphabet = kwargs.pop('alphabet', Hashids.ALPHABET)
        self.prefix = kwargs.pop('alphabet', "")

        source_field = kwargs.pop('source_field', None)
        if source_field:
            from hashid_field import HashidField, BigHashidField, HashidAutoField, BigHashidAutoField
            if isinstance(source_field, str):
                try:
                    app_label, model_name, field_name = source_field.split(".")
                except ValueError:
                    raise ValueError(self.usage_text)
                model = apps.get_model(app_label, model_name)
                source_field = model._meta.get_field(field_name)
            elif not isinstance(source_field, (HashidField, BigHashidField, HashidAutoField, BigHashidAutoField)):
                raise TypeError(self.usage_text)
            self.hashid_salt, self.hashid_min_length, self.hashid_alphabet, self.prefix = \
                source_field.salt, source_field.min_length, source_field.alphabet, source_field.prefix
        self._hashids = Hashids(salt=self.hashid_salt, min_length=self.hashid_min_length, alphabet=self.hashid_alphabet)
        super().__init__(**kwargs)

    def to_internal_value(self, data):
        try:
            value = super().to_internal_value(data)
            return Hashid(value, prefix=self.prefix, hashids=self._hashids)
        except ValueError:
            raise serializers.ValidationError("Invalid int or Hashid string")


class HashidSerializerCharField(HashidSerializerMixin, fields.CharField):
    def to_representation(self, value):
        return str(value)


class HashidSerializerIntegerField(HashidSerializerMixin, fields.IntegerField):
    def to_representation(self, value):
        return int(value)




from django import forms
from django.core import exceptions, checks
from django.db import models
from django.db.models import Field
from django.utils.translation import gettext_lazy as _
from django.contrib.admin import widgets as admin_widgets
from hashids import Hashids

from .lookups import HashidExactLookup, HashidIterableLookup
from .lookups import HashidGreaterThan, HashidGreaterThanOrEqual, HashidLessThan, HashidLessThanOrEqual
from .descriptor import HashidDescriptor
from .hashid import Hashid
from .conf import settings


def _alphabet_unique_len(alphabet):
    return len([x for i, x in enumerate(alphabet) if alphabet.index(x) == i])


class HashidFieldMixin(object):
    default_error_messages = {
        'invalid': _("'%(value)s' value must be a positive integer or a valid Hashids string."),
        'invalid_hashid': _("'%(value)s' value must be a valid Hashids string."),
    }
    exact_lookups = ('exact', 'iexact', 'contains', 'icontains')
    iterable_lookups = ('in',)
    passthrough_lookups = ('isnull',)
    comparison_lookups = {
        'gt': HashidGreaterThan,
        'gte': HashidGreaterThanOrEqual,
        'lt': HashidLessThan,
        'lte': HashidLessThanOrEqual,
    }

    def __init__(self, salt=settings.HASHID_FIELD_SALT,
                 min_length=settings.HASHID_FIELD_MIN_LENGTH,
                 alphabet=settings.HASHID_FIELD_ALPHABET,
                 allow_int_lookup=settings.HASHID_FIELD_ALLOW_INT_LOOKUP,
                 enable_hashid_object=settings.HASHID_FIELD_ENABLE_HASHID_OBJECT,
                 enable_descriptor=settings.HASHID_FIELD_ENABLE_DESCRIPTOR,
                 prefix="", *args, **kwargs):
        self.salt = salt
        self.min_length = min_length
        self.alphabet = alphabet
        if _alphabet_unique_len(self.alphabet) < 16:
            raise exceptions.ImproperlyConfigured("'alphabet' must contain a minimum of 16 unique characters")
        self._hashids = Hashids(salt=self.salt, min_length=self.min_length, alphabet=self.alphabet)
        self.allow_int_lookup = allow_int_lookup
        self.enable_hashid_object = enable_hashid_object
        self.enable_descriptor = enable_descriptor
        self.prefix = prefix
        super().__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        kwargs['min_length'] = self.min_length
        kwargs['alphabet'] = self.alphabet
        kwargs['prefix'] = self.prefix
        return name, path, args, kwargs

    def check(self, **kwargs):
        errors = super().check(**kwargs)
        errors.extend(self._check_alphabet_min_length())
        errors.extend(self._check_salt_is_set())
        return errors

    def _check_alphabet_min_length(self):
        if _alphabet_unique_len(self.alphabet) < 16:
            return [
                checks.Error(
                    "'alphabet' must contain a minimum of 16 unique characters",
                    hint="Add more unique characters to custom 'alphabet'",
                    obj=self,
                    id='HashidField.E001',
                )
            ]
        return []

    def _check_salt_is_set(self):
        if self.salt is None or self.salt == "":
            return [
                checks.Warning(
                    "'salt' is not set",
                    hint="Pass a salt value in your field or set settings.HASHID_FIELD_SALT",
                    obj=self,
                    id="HashidField.W001",
                )
            ]
        return []

    def encode_id(self, id):
        hashid = self.get_hashid(id)
        if self.enable_hashid_object:
            return hashid
        else:
            return str(hashid)

    def get_hashid(self, id):
        return Hashid(id, prefix=self.prefix, hashids=self._hashids)

    def from_db_value(self, value, expression, connection):
        if value is None:
            return value
        return self.encode_id(value)

    def get_lookup(self, lookup_name):
        if lookup_name in self.exact_lookups:
            return HashidExactLookup
        if lookup_name in self.iterable_lookups:
            return HashidIterableLookup
        if lookup_name in self.comparison_lookups:
            return self.comparison_lookups[lookup_name]
        if lookup_name in self.passthrough_lookups:
            return super().get_lookup(lookup_name)
        return None  # Otherwise, we don't allow lookups of this type

    def to_python(self, value):
        if isinstance(value, Hashid):
            return value
        if value is None:
            return value
        try:
            hashid = self.encode_id(value)
        except ValueError:
            raise exceptions.ValidationError(
                self.error_messages['invalid'],
                code='invalid',
                params={'value': value},
            )
        return hashid

    def get_prep_value(self, value):
        if value is None or value == '':
            return None
        if isinstance(value, Hashid):
            return value.id
        try:
            hashid = self.get_hashid(value)
        except ValueError:
            raise ValueError(self.error_messages['invalid'] % {'value': value})
        return hashid.id

    def contribute_to_class(self, cls, name, **kwargs):
        super().contribute_to_class(cls, name, **kwargs)
        # if callable(self.prefix):
        #     self.prefix = self.prefix(field_instance=self, model_class=cls, field_name=name, **kwargs)
        if self.enable_descriptor:
            descriptor = HashidDescriptor(field_name=self.attname, hashids=self._hashids, prefix=self.prefix,
                                          enable_hashid_object=self.enable_hashid_object)
            setattr(cls, self.attname, descriptor)


class HashidCharFieldMixin:
    def formfield(self, **kwargs):
        defaults = {'form_class': forms.CharField}
        defaults.update(kwargs)
        if defaults.get('widget') == admin_widgets.AdminIntegerFieldWidget:
            defaults['widget'] = admin_widgets.AdminTextInputWidget
        if defaults.get('widget') == admin_widgets.AdminBigIntegerFieldWidget:
            defaults['widget'] = admin_widgets.AdminTextInputWidget
        # noinspection PyCallByClass,PyTypeChecker
        return Field.formfield(self, **defaults)


class HashidField(HashidFieldMixin, HashidCharFieldMixin, models.IntegerField):
    description = "A Hashids obscured IntegerField"


class BigHashidField(HashidFieldMixin, HashidCharFieldMixin, models.BigIntegerField):
    description = "A Hashids obscured BigIntegerField"

    def __init__(self, min_length=settings.HASHID_FIELD_BIG_MIN_LENGTH, *args, **kwargs):
        super().__init__(min_length=min_length, *args, **kwargs)


class HashidAutoField(HashidFieldMixin, models.AutoField):
    description = "A Hashids obscured AutoField"


class BigHashidAutoField(HashidFieldMixin, models.AutoField):
    # This inherits from AutoField instead of BigAutoField so that DEFAULT_AUTO_FIELD doesn't throw an error
    description = "A Hashids obscured BigAutoField"

    def get_internal_type(self):
        return 'BigAutoField'

    def rel_db_type(self, connection):
        return models.BigIntegerField().db_type(connection=connection)

    def __init__(self, min_length=settings.HASHID_FIELD_BIG_MIN_LENGTH, *args, **kwargs):
        super().__init__(min_length=min_length, *args, **kwargs)


# Monkey patch Django REST Framework, if it's installed, to throw exceptions if fields aren't explicitly defined in
# ModelSerializers. Not doing so can lead to hard-to-debug behavior.
try:
    from rest_framework.serializers import ModelSerializer
    from hashid_field.rest import UnconfiguredHashidSerialField

    ModelSerializer.serializer_field_mapping[HashidField] = UnconfiguredHashidSerialField
    ModelSerializer.serializer_field_mapping[BigHashidField] = UnconfiguredHashidSerialField
    ModelSerializer.serializer_field_mapping[HashidAutoField] = UnconfiguredHashidSerialField
    ModelSerializer.serializer_field_mapping[BigHashidAutoField] = UnconfiguredHashidSerialField
except ImportError:
    pass



from .field import HashidField, BigHashidField, HashidAutoField, BigHashidAutoField
from .hashid import Hashid

__title__ = 'Django Hashid Field'
__version__ = "3.2.0"
__author__ = 'Nathan Shafer'
__license__ = 'MIT License'
__copyright__ = 'Copyright 2017-2021 Nathan Shafer'

# Version synonym
VERSION = __version__



from functools import total_ordering

from hashids import Hashids


def _is_int(number):
    """Returns whether a value is an integer."""
    try:
        return number == int(number)
    except ValueError:
        return False


def _is_uint(number):
    """Returns whether a value is an unsigned integer."""
    try:
        return number == int(number) and number >= 0
    except ValueError:
        return False


def _is_str(candidate):
    """Returns whether a value is a string."""
    return isinstance(candidate, str)


@total_ordering
class Hashid(object):
    def __init__(self, value, salt="", min_length=0, alphabet=Hashids.ALPHABET, prefix="", hashids=None):
        if hashids is None:
            self._salt = salt
            self._min_length = min_length
            self._alphabet = alphabet
            self._hashids = Hashids(salt=self._salt, min_length=self._min_length, alphabet=self._alphabet)
        else:
            self._hashids = hashids
            self._salt = hashids._salt
            self._min_length = hashids._min_length
            self._alphabet = hashids._alphabet

        self._prefix = str(prefix)

        if value is None:
            raise ValueError("id must be a positive integer or a valid Hashid string")

        # Check if `value` is an integer first as it is much faster than checking if a string is a valid hashid
        if _is_uint(value):
            self._id = value
            self._hashid = self.encode(value)
        elif _is_str(value):
            # `value` could be a string representation of an integer and not a hashid, but since the Hashids algorithm
            # requires a minimum of 16 characters in the alphabet, `int(value, base=10)` will always throw a ValueError
            # for a hashids string, as it's impossible to represent a hashids string with only chars [0-9].
            try:
                value = int(value, base=10)
            except (TypeError, ValueError):
                # We must assume that this string is a hashids representation.
                # Verify that it begins with the prefix, which could be the default ""
                if not value.startswith(self._prefix):
                    raise ValueError("value must begin with prefix {}".format(self._prefix))

                without_prefix = value[len(self._prefix):]
                _id = self.decode(without_prefix)
                if _id is None:
                    raise ValueError("id must be a positive integer or a valid Hashid string")
                else:
                    self._id = _id
                    self._hashid = without_prefix
            else:
                if not _is_uint(value):
                    raise ValueError("value must be a positive integer")

                # Finally, set our internal values
                self._id = value
                self._hashid = self.encode(value)
        elif isinstance(value, int) and value < 0:
            raise ValueError("value must be a positive integer")
        else:
            raise ValueError("value must be a positive integer or a valid Hashid string")

    @property
    def id(self):
        return self._id

    @property
    def hashid(self):
        return self._hashid

    @property
    def prefix(self):
        return self._prefix

    @property
    def hashids(self):
        return self._hashids

    def encode(self, id):
        return self._hashids.encode(id)

    def decode(self, hashid):
        ret = self._hashids.decode(hashid)
        if len(ret) == 1:
            return ret[0]
        else:
            return None

    def __repr__(self):
        return "Hashid({}): {}".format(self._id, str(self))

    def __str__(self):
        return self._prefix + self._hashid

    def __int__(self):
        return self._id

    def __long__(self):
        return int(self._id)

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return (
                self._id == other._id and
                self._hashid == other._hashid and
                self._prefix == other._prefix
            )
        if isinstance(other, str):
            return str(self) == other
        if isinstance(other, int):
            return int(self) == other
        return NotImplemented

    def __lt__(self, other):
        if isinstance(other, self.__class__):
            return self._id < other._id
        if isinstance(other, type(self._id)):
            return self._id < other
        return NotImplemented

    def __len__(self):
        return len(str(self))

    def __hash__(self):
        return hash(str(self))

    def __reduce__(self):
        return (self.__class__, (self._id, self._salt, self._min_length, self._alphabet, self._prefix, None))



import warnings

from django.conf import settings

setattr(settings, 'HASHID_FIELD_SALT', getattr(settings, 'HASHID_FIELD_SALT', ""))
setattr(settings, 'HASHID_FIELD_MIN_LENGTH', getattr(settings, 'HASHID_FIELD_MIN_LENGTH', 7))
setattr(settings, 'HASHID_FIELD_BIG_MIN_LENGTH', getattr(settings, 'HASHID_FIELD_BIG_MIN_LENGTH', 13))
setattr(settings, 'HASHID_FIELD_ALPHABET',
        getattr(settings, 'HASHID_FIELD_ALPHABET', "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890"))
setattr(settings, 'HASHID_FIELD_ALLOW_INT_LOOKUP', getattr(settings, 'HASHID_FIELD_ALLOW_INT_LOOKUP', False))
setattr(settings, 'HASHID_FIELD_LOOKUP_EXCEPTION', getattr(settings, 'HASHID_FIELD_LOOKUP_EXCEPTION', False))
setattr(settings, 'HASHID_FIELD_ENABLE_HASHID_OBJECT', getattr(settings, 'HASHID_FIELD_ENABLE_HASHID_OBJECT', True))
setattr(settings, 'HASHID_FIELD_ENABLE_DESCRIPTOR', getattr(settings, 'HASHID_FIELD_ENABLE_DESCRIPTOR', True))




"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
import os
import re
import shutil
import sys

from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open

here = os.path.abspath(os.path.dirname(__file__))


# Get the VERSION from the __init__ file.
def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(here, package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version('hashid_field')


# Get the long description from the README file
with open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()


if sys.argv[-1] == 'publish':
    if os.system("pip freeze | grep twine"):
        print("twine not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    os.system('python setup.py sdist bdist_wheel')
    os.system('twine upload dist/*')
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    shutil.rmtree('dist')
    shutil.rmtree('build')
    shutil.rmtree('django_hashid_field.egg-info')
    sys.exit()


setup(
    name='django-hashid-field',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=version,

    description='A Hashids obfuscated Django Model Field',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/nshafer/django-hashid-field',

    # Author details
    author='Nathan Shafer',
    author_email='nate-pypi@lotech.org',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',

        # Django
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
    ],

    # What does your project relate to?
    keywords='django hashids hashid',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests', 'sandbox']),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['Django>=1.11', 'hashids>=1.2.0'],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    # extras_require={
    #     'dev': ['check-manifest'],
    #     'test': ['coverage'],
    # },

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    # package_data={
    #     'sample': ['package_data.dat'],
    # },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=[('my_data', ['data/data_file'])],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    # entry_points={
    #     'console_scripts': [
    #         'sample=sample:main',
    #     ],
    # },
)
